using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.IO;

namespace DesktopManager
{
    public static class DesktopServer
    {
        public static int Port = 13666;
        private static Thread listen = null;
        private static Thread work = null;
        private static TcpListener listener = null;
        private static List<TcpClient> clients = new List<TcpClient>();
        private static TcpClient client = null;
        private static NetworkStream stream = null;
        private static byte[] data = new byte[2 * 1024 * 1024];

        public static void Start()
        {
            if ( listen != null )
            {
                listen.Abort();
                try
                {
                    listener.Stop();
                }
                catch { }
                try
                {
                    work.Abort();
                }
                catch { }
                clients.Clear();
            } // if

            listen = new Thread( DesiredStart );
            listen.Priority = ThreadPriority.Highest;
            listen.Start();
        }

        public static void Stop()
        {
            if ( listen != null )
            {
                listen.Abort();
                listener.Stop();
                try
                {
                    work.Abort();
                }
                catch { }
                clients.Clear();
            } // if
        }

        private static void DesiredStart()
        {
            work = new Thread( DesiredRespond );
            work.Priority = ThreadPriority.AboveNormal;
            work.Start();

            listener = new TcpListener( IPAddress.Any, Port );
            listener.Start();
            while ( true )
            {
                TcpClient client = listener.AcceptTcpClient();
                lock ( clients )
                    clients.Add( client );
                work.Resume();
            } // while
        }

        private static void DesiredRespond()
        {
            while ( true )
            {
                if ( clients.Count == 0 )
                    Thread.CurrentThread.Suspend();
                else
                {
                    TcpClient clnt = null;
                    lock ( clients )
                    {
                        clnt = clients[0];
                        clients.RemoveAt( 0 );
                    } // lock
                    client = clnt;
                    client.ReceiveBufferSize = data.Length;
                    client.ReceiveTimeout = 4000;
                    client.SendBufferSize = data.Length;
                    client.SendTimeout = 4000;
                    stream = client.GetStream();
                    Thread t = new Thread( Responder );
                    t.Priority = ThreadPriority.Normal;
                    t.Start();
                } // if
            } // while
        }

        private static void Responder()
        {
            TcpClient clnt = client;
            client = null;
            NetworkStream strm = stream;
            stream = null;
            MemoryStream ms = new MemoryStream();

            try
            {
                Bitmap bitmap = new Bitmap( Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height );
                Graphics g = Graphics.FromImage( bitmap );

                while ( true )
                {
                    g.CopyFromScreen( 0, 0, 0, 0, Screen.PrimaryScreen.Bounds.Size );
                    g.Save();
                    bitmap.Save( ms, ImageFormat.Jpeg );

                    ms.Seek( 0, SeekOrigin.Begin );
                    ms.Read( data, 0, (int)ms.Length );
                    strm.Write( data, 0, (int)ms.Length );
                    ms.SetLength( 0 );
                    Thread.Sleep( 40 );
                } // while
            }
            catch
            {
                try
                {
                    strm.Close();
                }
                catch { }

                try
                {
                    clnt.Close();
                }
                catch { }
            } // try
        }
    }
}