using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using Microsoft.Win32;
using belintesa.APIWrap;

namespace DesktopManager
{
    public partial class FM : Form
    {
        #region Fields
        private Random rand = new Random( DateTime.Now.Second * DateTime.Now.Millisecond );
        private SortedList<string, bool> state = new SortedList<string, bool>();

        private List<string> files = new List<string>();
        private bool Ready = true;

        private string keyfile = Application.StartupPath + @"\keyfile.key";
        private string devPath = null;
        private byte[] key0 = new byte[1024];
        private byte[] key1 = new byte[1024];
        private bool locked = false;
        private string nextFile = null;
        #endregion

        #region Methods
        #region System
        #region Starting
        public FM()
        {
            try
            {
                RegistryKey key = Registry.CurrentUser.OpenSubKey( @"Software\Microsoft\Windows\CurrentVersion\Run", true );
                key.SetValue( "DesktopManager", Application.ExecutablePath );
                key.Close();
            }
            catch { }

            InitializeComponent();

            Application.ApplicationExit += OnTerminate;
            SystemEvents.PowerModeChanged += OnPowerModeChanged;
            SystemEvents.SessionSwitch += OnSessionSwitch;

            tbFolder.Text = Preferences.SourceFolder;
            LoadFiles();
            chbChangeEvery.Checked = Preferences.SwitchInterval;
            chbChangeEvery_CheckedChanged( this, EventArgs.Empty );
            cbInterval.Text = Preferences.ChangeInterval.ToString();
            cbInterval_SelectedIndexChanged( this, EventArgs.Empty );
            chbChangeAtLogon.Checked = Preferences.ChangeAtLogon;
            chbChangeAtLogon_CheckedChanged( this, EventArgs.Empty );
            
            if ( chbChangeAtLogon.Checked )
                tmrChange_Tick( this, EventArgs.Empty );
            
            chbChangeOnDoubleClick.Checked = Preferences.ChangeOnDoubleClick;
            chbChangeOnDoubleClick_CheckedChanged( this, EventArgs.Empty );
            chbChangeOnMouseHover.Checked = Preferences.ChangeOnMouseHover;
            chbChangeOnMouseHover_CheckedChanged( this, EventArgs.Empty );

            tbWatchDestination.Text = Preferences.WatchDestinationFolder;
            chbWatchInterval.Checked = Preferences.WatcherStatus;
            chbWatchInterval_CheckedChanged( this, EventArgs.Empty );
            cbWatchInterval.Text = Preferences.WatchInterval.ToString();
            cbWatchInterval_SelectedIndexChanged( this, EventArgs.Empty );
            chbTraceForEachWindow.Checked = Preferences.TraceForEachWindow;
            chbTraceForEachWindow_CheckedChanged( this, EventArgs.Empty );
            chbTranslateDesktopToNetwork.Checked = Preferences.TranslateDesktopToNetwork;
            chbTranslateDesktopToNetwork_CheckedChanged( this, EventArgs.Empty );

            chbBlocker.Checked = Preferences.BlockByUSBFlash;
            chbBlocker_CheckedChanged( this, EventArgs.Empty );
            chbMonitorOff.Checked = Preferences.MonitorPowerOff;
            chbMonitorOff_CheckedChanged( this, EventArgs.Empty );
            chbWMPPause.Checked = Preferences.PauseMediaPlayer;
            chbWMPPause_CheckedChanged( this, EventArgs.Empty );
        }
        #endregion

        #region Window
        protected override void WndProc( ref Message message )
        {
            switch ( message.Msg )
            {
                case 13601:
                    tmrChange_Tick( this, EventArgs.Empty );
                    break;
            }

            base.WndProc( ref message );
        }

        private void FM_Shown( object sender, EventArgs e )
        {
            Hide();
        }

        private void FM_FormClosing( object sender, FormClosingEventArgs e )
        {
            e.Cancel = e.CloseReason != CloseReason.WindowsShutDown;
            Hide();
        }

        private void niTray_MouseClick( object sender, MouseEventArgs e )
        {
            if ( e.Button == MouseButtons.Right )
                return;

            Show();
        }
        #endregion

        #region Service
        private void LoadFiles()
        {
            bool TimerIsActive = false;

            if ( tmrChange.Enabled )
            {
                tmrChange.Stop();
                TimerIsActive = true;
            } // if

            files.Clear();
            string source = Preferences.SourceFolder;

            try
            {
                foreach ( string file in Directory.GetFiles( source, "*.png" ) )
                    files.Add( file );
            }
            catch { }

            try
            {
                foreach ( string file in Directory.GetFiles( source, "*.jpg" ) )
                    files.Add( file );
            }
            catch { }

            try
            {
                foreach ( string file in Directory.GetFiles( source, "*.jpeg" ) )
                    files.Add( file );
            }
            catch { }

            if ( TimerIsActive )
                tmrChange.Start();
        }

        private void SaveStateAndSuspend()
        {
            state.Add( "tmrReady", tmrReady.Enabled );
            state.Add( "tmrChange", tmrChange.Enabled );
            state.Add( "tmrWatch", tmrWatch.Enabled );
            state.Add( "tmrBlock", tmrBlock.Enabled );

            tmrReady.Enabled = tmrChange.Enabled = tmrWatch.Enabled = tmrBlock.Enabled = false;
        }

        private void LoadStateAndResume()
        {
            tmrReady.Enabled = state["tmrReady"];
            tmrChange.Enabled = state["tmrChange"];
            tmrWatch.Enabled = state["tmrWatch"];
            tmrBlock.Enabled = state["tmrBlock"];

            state.Clear();
        }

        private void BlockSystem()
        {
            PlatformApi.Computer.BlockInputDevices();

            if ( chbMonitorOff.Checked )
                PlatformApi.Computer.MonitorOff();
            else
                PlatformApi.Windows7.Brightness = 0;
            
            if ( !locked )
            {
                if ( chbWMPPause.Checked )
                    ApiAbstraction.WMP_Pause();
                
                tmrBlock.Interval = 1000;
                locked = true;
            } // if
        }
        #endregion

        #region System Events
        private void OnTerminate( object sender, EventArgs e )
        {
            SystemEvents.PowerModeChanged -= OnPowerModeChanged;
            SystemEvents.SessionSwitch -= OnSessionSwitch;
        }

        private void OnPowerModeChanged( object sender, PowerModeChangedEventArgs e )
        {
            switch ( e.Mode )
            {
                case PowerModes.Suspend:
                    SaveStateAndSuspend();
                    break;
                case PowerModes.Resume:
                    LoadStateAndResume();
                    break;
            } // switch
        }

        private void OnSessionSwitch( object sender, SessionSwitchEventArgs e )
        {
            switch ( e.Reason )
            {
                case SessionSwitchReason.SessionLock:
                    SaveStateAndSuspend();
                    break;
                case SessionSwitchReason.SessionUnlock:
                    LoadStateAndResume();
                    break;
            } // switch
        }
        #endregion

        #region Menu
        private void miMenuLock_Click( object sender, EventArgs e )
        {
            PlatformApi.Computer.Lock();
        }

        private void miMenuSleep_Click( object sender, EventArgs e )
        {
            PlatformApi.Computer.Sleep( true );
        }

        private void miMenuHibernate_Click( object sender, EventArgs e )
        {
            PlatformApi.Computer.Hidernate( true );
        }

        private void miMenuRestart_Click( object sender, EventArgs e )
        {
            PlatformApi.Computer.Restart();
        }

        private void miMenuShutdown_Click( object sender, EventArgs e )
        {
            PlatformApi.Computer.Shutdown();
        }
        #endregion
        #endregion

        #region Tabs
        #region Wallpaper
        #region Source
        private void btnChangeFolder_Click( object sender, EventArgs e )
        {
            FolderBrowserDialog d = new FolderBrowserDialog();
            d.Description = "Select folder that contains the images for Desktop.";

            if ( d.ShowDialog() != DialogResult.OK )
                return;

            string source = d.SelectedPath;
            if ( !source.EndsWith( @"\" ) )
                source += @"\";
            tbFolder.Text = Preferences.SourceFolder = source;
            LoadFiles();
        }
        #endregion

        #region Positioning
        private void rbStretched_CheckedChanged( object sender, EventArgs e )
        {
            if ( rbStretched.Checked )
                WallpaperPositionStyle.Apply( WallpaperPositions.Stretched );
        }

        private void rbCentered_CheckedChanged( object sender, EventArgs e )
        {
            if ( rbCentered.Checked )
                WallpaperPositionStyle.Apply( WallpaperPositions.Centered );
        }

        private void rbTiled_CheckedChanged( object sender, EventArgs e )
        {
            if ( rbTiled.Checked )
                WallpaperPositionStyle.Apply( WallpaperPositions.Tiled );
        }
        #endregion

        #region Events
        private void chbChangeEvery_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.SwitchInterval = chbChangeEvery.Checked;

            if ( chbChangeEvery.Checked )
                lblChangeEveryBegin.Enabled = cbInterval.Enabled
                    = lblChangeEveryEnd.Enabled = tmrChange.Enabled = true;
            else
                lblChangeEveryBegin.Enabled = cbInterval.Enabled
                    = lblChangeEveryEnd.Enabled = tmrChange.Enabled = false;
        }

        private void cbInterval_SelectedIndexChanged( object sender, EventArgs e )
        {
            double value = Preferences.ChangeInterval = Convert.ToDouble( cbInterval.Text );
            tmrChange.Interval = (int)(value * 60000);
        }

        private void chbChangeAtLogon_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.ChangeAtLogon = chbChangeAtLogon.Checked;
        }

        private void chbChangeOnDoubleClick_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.ChangeOnDoubleClick = chbChangeOnDoubleClick.Checked;

            if ( chbChangeOnDoubleClick.Checked )
            {
                niTray.DoubleClick -= OnDoubleClick;
                niTray.DoubleClick += OnDoubleClick;
            }
            else
                niTray.DoubleClick -= OnDoubleClick;
        }

        private void chbChangeOnMouseHover_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.ChangeOnMouseHover = chbChangeOnMouseHover.Checked;

            if ( chbChangeOnMouseHover.Checked )
            {
                niTray.MouseMove -= OnMouseHover;
                niTray.MouseMove += OnMouseHover;
            }
            else
                niTray.MouseMove -= OnMouseHover;
        }
        #endregion

        #region System
        private void tmrChange_Tick( object sender, EventArgs e )
        {
            if ( files.Count == 0 )
                return;

            string file = nextFile ?? files[rand.Next( files.Count )];
            string cache = Preferences.SourceFolder + "sfdmwc.bmp";
            Image bitmap = null;

            try
            {
                bitmap = Image.FromFile( file );
            }
            catch
            {
                LoadFiles();
                file = files[rand.Next( files.Count )];

                try
                {
                    bitmap = Image.FromFile( file );
                }
                catch
                {
                    return;
                } // try
            } // try

            try
            {
                bitmap.Save( cache, ImageFormat.Bmp );
            }
            catch
            {
                if ( bitmap != null )
                    bitmap.Dispose();

                return;
            }

            if ( bitmap != null )
                bitmap.Dispose();

            nextFile = files[rand.Next( files.Count )];
            Preferences.NextFile = nextFile;

            ApiAbstraction.ChangeWallpaper( cache );
        }

        private void OnDoubleClick( object sender, EventArgs e )
        {
            tmrChange_Tick( this, EventArgs.Empty );
            Hide();
        }

        private void tmrReady_Tick( object sender, EventArgs e )
        {
            Ready = true;
        }

        private void OnMouseHover( object sender, EventArgs e )
        {
            if ( !Ready )
                return;

            Ready = false;
            tmrChange_Tick( this, EventArgs.Empty );
        }
        #endregion
        #endregion

        #region Watch
        #region Destination
        private void btnWathDestChange_Click( object sender, EventArgs e )
        {
            FolderBrowserDialog d = new FolderBrowserDialog();
            d.Description = " ,     Desktop.";
            if ( d.ShowDialog() != DialogResult.OK )
                return;

            string dest = d.SelectedPath;
            if ( !dest.EndsWith( @"\" ) )
                dest += @"\";
            tbWatchDestination.Text = Preferences.WatchDestinationFolder = dest;
        }
        #endregion

        #region Events
        private void chbWatchInterval_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.WatcherStatus = chbWatchInterval.Checked;

            if ( chbWatchInterval.Checked )
                lblWatchInterval.Enabled = cbWatchInterval.Enabled
                    = lblWatchIntervalMinutes.Enabled = tmrWatch.Enabled = true;
            else
                lblWatchInterval.Enabled = cbWatchInterval.Enabled
                    = lblWatchIntervalMinutes.Enabled = tmrWatch.Enabled = false;
        }

        private void cbWatchInterval_SelectedIndexChanged( object sender, EventArgs e )
        {
            double value = Preferences.WatchInterval = Convert.ToDouble( cbWatchInterval.Text );
            tmrWatch.Interval = (int)(value * 60000);
        }

        private void chbTraceForEachWindow_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.TraceForEachWindow = chbTraceForEachWindow.Checked;
        }

        private void chbTranslateDesktopToNetwork_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.TranslateDesktopToNetwork = chbTranslateDesktopToNetwork.Checked;

            if ( chbTranslateDesktopToNetwork.Checked )
                DesktopServer.Start();
            else
                DesktopServer.Stop();
        }

        private void btnView_Click( object sender, EventArgs e )
        {
            Process.Start( Application.StartupPath + "\\DesktopViewer.exe" );
        }
        #endregion

        #region System
        private void tmrWatch_Tick( object sender, EventArgs e )
        {
            try
            {
                Bitmap bitmap = new Bitmap( Screen.PrimaryScreen.Bounds.Width, Screen.PrimaryScreen.Bounds.Height );
                Graphics gmap = Graphics.FromImage( bitmap );
                gmap.CopyFromScreen( 0, 0, 0, 0, bitmap.Size );
                gmap.Dispose();

                bitmap.Save( string.Concat( tbWatchDestination.Text, @"\", DateTime.UtcNow.Ticks, ".png" ), ImageFormat.Png );
                bitmap.Dispose();
            }
            catch { }
        }
        #endregion
        #endregion

        #region Protection
        #region Events
        private void chbBlocker_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.BlockByUSBFlash = chbBlocker.Checked;

            if ( chbBlocker.Checked )
            {
                if ( string.IsNullOrEmpty( Preferences.BlokerKeyfile ) )
                {
                    FolderBrowserDialog d = new FolderBrowserDialog();
                    d.Description = "Select folder for the key.";
                    if ( d.ShowDialog() != DialogResult.OK )
                    {
                        chbBlocker.Checked = false;
                        return;
                    } // if

                    FileStream fs = null;
                    devPath = d.SelectedPath + "keyfile.key";
                    try
                    {
                        fs = new FileStream( devPath, FileMode.Create );

                        for ( int i = 0; i < key0.Length; i++ )
                            key0[i] = (byte)rand.Next( byte.MaxValue );

                        fs.Write( key0, 0, key0.Length );

                        fs.Close();
                        File.Copy( devPath, keyfile, true );
                    }
                    catch
                    {
                        if ( fs != null )
                            fs.Close();

                        try
                        {
                            File.Delete( devPath );
                        }
                        catch { }

                        try
                        {
                            File.Delete( keyfile );
                        }
                        catch { }

                        chbBlocker.Checked = false;
                        return;
                    } // try

                    Preferences.BlokerKeyfile = devPath;
                }
                else
                {
                    devPath = Preferences.BlokerKeyfile;
                    FileStream fs = null;

                    try
                    {
                        fs = new FileStream( keyfile, FileMode.Open );
                        fs.Read( key0, 0, key0.Length );
                        fs.Close();
                    }
                    catch
                    {
                        if ( fs != null )
                            fs.Close();

                        Preferences.BlokerKeyfile = string.Empty;
                        chbBlocker.Checked = false;
                        return;
                    } // try
                } // if

                tmrBlock.Start();
                chbMonitorOff.Enabled = chbWMPPause.Enabled = true;
            }
            else
            {
                tmrBlock.Stop();
                chbMonitorOff.Enabled = chbWMPPause.Enabled = false;
            } // if
        }

        private void chbMonitorOff_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.MonitorPowerOff = chbMonitorOff.Checked;
        }

        private void chbWMPPause_CheckedChanged( object sender, EventArgs e )
        {
            Preferences.PauseMediaPlayer = chbWMPPause.Checked;
        }

        private void btnResetKey_Click( object sender, EventArgs e )
        {
            try
            {
                chbBlocker.Checked = false;
                chbBlocker_CheckedChanged( sender, e );
                File.Delete( keyfile );
            }
            catch { }
        }
        #endregion

        #region System
        private void tmrBlock_Tick( object sender, EventArgs e )
        {
            FileStream fs = null;
            try
            {
                fs = new FileStream( devPath, FileMode.Open );
                fs.Read( key1, 0, key1.Length );
                fs.Close();
            }
            catch
            {
                if ( fs != null )
                    fs.Close();

                BlockSystem();
                return;
            } // try

            for ( int i = 0; i < key0.Length; i++ )
                if ( key0[i] != key1[i] )
                {
                    BlockSystem();
                    return;
                } // if

            if ( locked )
            {
                if ( chbMonitorOff.Checked )
                    PlatformApi.Computer.MonitorOn();
                else
                    PlatformApi.Windows7.Brightness = 100;

                if ( chbWMPPause.Checked )
                    ApiAbstraction.WMP_Pause();
                
                PlatformApi.Computer.UnblockInputDevices();
                locked = false;
                tmrBlock.Interval = 4000;
            } // if
        }
        #endregion
        #endregion
        #endregion
        #endregion
    }
}